Improve this doc

# Internet Explorer Compatibility

Note: AngularJS 1.3 is dropping support for IE8. Read more about it on our blog. AngularJS 1.2 will continue to support IE8, but the core team does not plan to spend time addressing issues specific to IE8 or earlier.

This document describes the Internet Explorer (IE) idiosyncrasies when dealing with custom HTML attributes and tags. Read this document if you are planning on deploying your Angular application on IE8 or earlier.

The project currently supports and will attempt to fix bugs for IE9 and above. The continuous integration server runs all the tests against IE9, IE10, and IE11. See Travis CI and ci.angularjs.org.

We do not run tests on IE8 and below. A subset of the AngularJS functionality may work on these browsers, but it is up to you to test and decide whether it works for your particular app.

## Short Version

To make your Angular application work on IE please make sure that:

1. You polyfill JSON.stringify for IE7 and below. You can use JSON2 or JSON3 polyfills for this.

         <!doctype html>
         <html xmlns:ng="http://angularjs.org">
           <head>
             <!--[if lte IE 7]>
               <script src="/path/to/json2.js"></script>
             <![endif]-->
           </head>
           <body>
             ...
           </body>
         </html>

2. add `id="ng-app"` to the root element in conjunction with `ng-app` attribute

         <!doctype html>
         <html xmlns:ng="http://angularjs.org" id="ng-app" ng-app="optionalModuleName">
           ...
         </html>

3. you do not use custom element tags such as `<ng:view>` (use the attribute version `<div ng-view>` instead), or

4. if you do use custom element tags, then you must take these steps to make IE 8 and below happy:

         <!doctype html>
         <html xmlns:ng="http://angularjs.org" id="ng-app" ng-app="optionalModuleName">
           <head>
             <!--[if lte IE 8]>
               <script>
                 document.createElement('ng-include');
                 document.createElement('ng-pluralize');
                 document.createElement('ng-view');

                 // Optionally these for CSS
                 document.createElement('ng:include');
                 document.createElement('ng:pluralize');
                 document.createElement('ng:view');
               </script>
             <![endif]-->
           </head>
           <body>
             ...
           </body>
         </html>

5. Use `ng-style` tags instead of `style="{{ someCss }}"`. The later works in Chrome and Firefox but does not work in Internet Explorer <= 11 (the most recent version at time of writing).

The important parts are:

- `xmlns:ng` - namespace - you need one namespace for each custom tag you are planning on using.

- `document.createElement(yourTagName)` - creation of custom tag names - Since this is an issue only for older version of IE you need to load it conditionally. For each tag which does not have namespace and which is not defined in HTML you need to pre-declare it to make IE happy.

## Long Version

IE has issues with element tag names which are not standard HTML tag names. These fall into two categories, and each category has its own fix.

- If the tag name starts with `my:` prefix then it is considered an XML namespace and must have corresponding namespace declaration on `<html xmlns:my="ignored">`

- If the tag has no `:` but it is not a standard HTML tag, then it must be pre-created using `document.createElement('my-tag')`

- If you are planning on styling the custom tag with CSS selectors, then it must be pre-created using `document.createElement('my-tag')` regardless of XML namespace.

## The Good News

The good news is that these restrictions only apply to element tag names, and not to element attribute names. So this requires no special handling in IE: `<div my-tag your:tag></div>`.

## What happens if I fail to do this?

Suppose you have HTML with unknown tag `mytag` (this could also be `my:tag` or `my-tag` with same result):

      <html>
        <body>
          <mytag>some text</mytag>
        </body>
      </html>

It should parse into the following DOM:

    #document
      +- HTML
         +- BODY
            +- mytag
               +- #text: some text

The expected behavior is that the `BODY` element has a child element `mytag`, which in turn has the text `some text`.

But this is not what IE does (if the above fixes are not included):

    #document
      +- HTML
         +- BODY
            +- mytag
            +- #text: some text
            +- /mytag

In IE, the behavior is that the `BODY` element has three children:

1. A self closing `mytag`. Example of self closing tag is `<br/>`. The trailing `/` is optional, but the `<br>` tag is not allowed to have any children, and browsers consider `<br>some text</br>` as three siblings not a `<br>` with `some text` as child.

2. A text node with `some text`. This should have been a child of `mytag` above, not a sibling.

3. A corrupt self closing `/mytag`. This is corrupt since element names are not allowed to have the `/` character. Furthermore this closing element should not be part of the DOM since it is only used to delineate the structure of the DOM.

## CSS Styling of Custom Tag Names

To make CSS selectors work with custom elements, the custom element name must be pre-created with `document.createElement('my-tag')` regardless of XML namespace.

      <html xmlns:ng="needed for ng: namespace">
        <head>
          <!--[if lte IE 8]>
            <script>
              // needed to make ng-include parse properly
              document.createElement('ng-include');

              // needed to enable CSS reference
              document.createElement('ng:view');
            </script>
          <![endif]-->
          <style>
            ng\:view {
              display: block;
              border: 1px solid red;
            }

            ng-include {
              display: block;
              border: 1px solid blue;
            }
          </style>
        </head>
        <body>
          <ng:view></ng:view>
          <ng-include></ng-include>
          ...
        </body>
      </html>